/* ------------------------------------------------------------------
 * Un exemple d'implmentation du cache d'un fichier afin d'explorer
 * l'effet des algorithmes de gestion et de remplacement
 * ------------------------------------------------------------------
 * Jean-Paul Rigault
 * ------------------------------------------------------------------
 * Implmentation du cache : spcification
 * ------------------------------------------------------------------
 */

#ifndef _LOW__CACHE_H_
#define _LOW_CACHE_H_

#include <stdio.h>
#include <stdlib.h>

#include "cache.h"

/* Entte de chaque bloc
* ----------------------
*/
struct Cache_Block_Header
{
    unsigned int flags; 	/* Indicateurs d'tat */
    int ibfile;			/* Index de ce block dans le fichier */
    int ibcache;		/* Index de ce block dans le cache */
    char *data; 		/* Les donnes */
};

/* Valeur des flags (indpendants de la stratgie) */
#define VALID 0x1 
#define MODIF 0x2

/* Le cache lui-mme
* ------------------
*/
struct Cache
{
    char *file;		    	/* Nom du fichier */
    FILE *fp;			/* Pointeur sur fichier */
    unsigned int nblocks;	/* Nb de blocs dans le cache */
    unsigned int nrecords;	/* Nombre d'enregistrements dans chaque bloc */
    size_t recordsz;		/* Taille d'un enregistrement */
    size_t blocksz;		/* Taille d'un bloc */
    unsigned int nderef;	/* priode de drfrenage pour NUR */
    void *pstrategy;		/* Structure de donnes dpendant de la stratgie */
    struct Cache_Instrument instrument;/* Instrumentation du cache */
    struct Cache_Block_Header *pfree;/* Premier bloc libre (invalide) */
    struct Cache_Block_Header *headers;/* Les donnes elles-mmes */
};

/* Frquence de synchronisation (un sync() tous les NSYNC accs) */
#define NSYNC 1000

/* Recherche d'un bloc libre (invalide) */

struct Cache_Block_Header *Get_Free_Block(struct Cache *pcache);

/* Macros de calcul d'adresse
* ---------------------------
*/

/* Adresse en octets de l'enregistrement d'indice-fichier ind dans le bloc point papb */
#define ADDR(pcache, ind, pb) \
    ((pb)->data + ((ind) % (pcache)->nrecords) * (pcache)->recordsz)

/* Adresse (en octets) du bloc d'index-fichier  ibfile dans le cache */
#define DADDR(pcache, ibfile) ((ibfile) * (pcache)->blocksz) 

#endif /* _LOW_CACHE_H_ */
